import {
	world,
	EntityItemComponent,
	GameMode,
	ItemEnchantableComponent,
	EquipmentSlot,
	Entity,
	ItemStack,
	EntityEquippableComponent,
	system
} from "@minecraft/server";
import {
	getScore,
	setScore
} from "./lib/main"

export class MinecraftEffectTypes {
	static Absorption = "absorption"
	static BadOmen = "bad_omen"
	static Blindness = "blindness"
	static ConduitPower = "conduit_power"
	static Darkness = "darkness"
	static Empty = "empty"
	static FatalPoison = "fatal_poison"
	static FireResistance = "fire_resistance"
	static Haste = "haste"
	static HealthBoost = "health_boost"
	static Hunger = "hunger"
	static InstantDamage = "instant_damage"
	static InstantHealth = "instant_health"
	static Invisibility = "invisibility"
	static JumpBoost = "jump_boost"
	static Levitation = "levitation"
	static MiningFatigue = "mining_fatigue"
	static Nausea = "nausea"
	static NightVision = "night_vision"
	static Poison = "poison"
	static Regeneration = "regeneration"
	static Resistance = "resistance"
	static Saturation = "saturation"
	static SlowFalling = "slow_falling"
	static Slowness = "slowness"
	static Speed = "speed"
	static Strength = "strength"
	static VillageHero = "village_hero"
	static WaterBreathing = "water_breathing"
	static Weakness = "weakness"
	static Wither = "wither"
	static WindCharged = "wind_charged"
	static Oozing = "oozing"
	static Infested = "infested"
	static Weaving = "weaving"
}

const config = {
	armorSets: [{
		name: "Infernal Armor",
		items: [{
			id: "oresplus:infernal_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:infernal_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:infernal_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:infernal_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.FireResistance,
			amplifier: 0,
			duration: 1000000
		}]
	},
	{
		name: "Angelic Armor",
		items: [{
			id: "oresplus:angelic_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:angelic_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:angelic_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:angelic_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.VillageHero,
			amplifier: 2,
			duration: 1000000
		}]
	},
	{
		name: "End Armor",
		items: [{
			id: "oresplus:end_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:end_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:end_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:end_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.SlowFalling,
			amplifier: 0,
			duration: 1000000
		}]
	},
	{
		name: "Ocean Armor",
		items: [{
			id: "oresplus:ocean_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:ocean_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:ocean_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:ocean_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.ConduitPower,
			amplifier: 2,
			duration: 1000000
		}]
	},
	{
		name: "Desert Armor",
		items: [{
			id: "oresplus:desert_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:desert_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:desert_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:desert_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.Speed,
			amplifier: 2,
			duration: 1000000
		}]
	},
	{
		name: "Spinel Armor",
		items: [{
			id: "oresplus:spinel_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:spinel_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:spinel_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:spinel_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.HealthBoost,
			amplifier: 5,
			duration: 1000000
		}]
	},
	{
		name: "Steel Armor",
		items: [{
			id: "oresplus:steel_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:steel_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:steel_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:steel_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.Slowness,
			amplifier: 1,
			duration: 1000000
		}]
	},
	{
		name: "Steel Armor",
		items: [{
			id: "oresplus:steel_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:steel_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:steel_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:steel_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.Resistance,
			amplifier: 1,
			duration: 1000000
		}]
	},
	{
		name: "Jade Armor",
		items: [{
			id: "oresplus:jade_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:jade_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:jade_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:jade_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.JumpBoost,
			amplifier: 4,
			duration: 1000000
		}]
	},
	{
		name: "Jade Armor",
		items: [{
			id: "oresplus:jade_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:jade_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:jade_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:jade_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.Speed,
			amplifier: 1,
			duration: 1000000
		}]
	},
	{
		name: "Cryo Armor",
		items: [{
			id: "oresplus:cryo_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:cryo_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:cryo_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:cryo_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.Slowness,
			amplifier: 0,
			duration: 1000000
		}]
	},
	{
		name: "Ambar Armor",
		items: [{
			id: "oresplus:ambar_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:ambar_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:ambar_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:ambar_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.Regeneration,
			amplifier: 1,
			duration: 1000000
		}]
	},
	{
		name: "Heillish Armor",
		items: [{
			id: "oresplus:heillish_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:heillish_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:heillish_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:heillish_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.HealthBoost,
			amplifier: 0,
			duration: 1000000
		}]
	},
	{
		name: "Heillish Armor",
		items: [{
			id: "oresplus:heillish_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:heillish_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:heillish_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:heillish_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.Speed,
			amplifier: 1,
			duration: 1000000
		}]
	},
	{
		name: "Heillish Armor",
		items: [{
			id: "oresplus:heillish_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:heillish_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:heillish_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:heillish_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.Strength,
			amplifier: 1,
			duration: 1000000
		}]
	},
	{
		name: "Pyrite Armor",
		items: [{
			id: "oresplus:pyrite_helmet",
			slot: EquipmentSlot.Head
		}, {
			id: "oresplus:pyrite_chestplate",
			slot: EquipmentSlot.Chest
		}, {
			id: "oresplus:pyrite_leggings",
			slot: EquipmentSlot.Legs
		}, {
			id: "oresplus:pyrite_boots",
			slot: EquipmentSlot.Feet
		}],
		effects: [{
			type: MinecraftEffectTypes.Resistance,
			amplifier: 0,
			duration: 1000000
		}]
	}]
};

function hasFullArmorSet(player, armorSet) {
	const equipComponent = player.getComponent(EntityEquippableComponent.componentId);
	return armorSet.items.every(item => {
		const equipment = equipComponent.getEquipment(item.slot);
		return equipment && equipment.typeId === item.id;
	});
}

const previousStates = {};
system.runInterval(() => {
	world.getPlayers().forEach(player => {
		const playerId = player.id;
		const currentStates = {};
		config.armorSets.forEach(armorSet => {
			const isEquipped = hasFullArmorSet(player, armorSet);
			currentStates[armorSet.name] = isEquipped;
			const wasEquipped = previousStates[playerId]?.[armorSet.name] || false;
			if (isEquipped && !wasEquipped) {
				const effect = armorSet.effects[0];
				if (effect) {
					player.addEffect(effect.type, effect.duration, {
						amplifier: effect.amplifier,
						showParticles: false
					});
				}
			} else if (!isEquipped && wasEquipped) {
				const effect = armorSet.effects[0];
				if (effect) {
					player.removeEffect(effect.type);
				}
			}
		});

		previousStates[playerId] = currentStates;
	});
}, 20);