import { world, ItemStack } from '@minecraft/server'

world.afterEvents.entityDie.subscribe(e => {
    const { deadEntity, damageSource } = e;

    if (!damageSource.damagingEntity || damageSource.damagingEntity.typeId !== 'minecraft:player') return;
    const player = damageSource.damagingEntity;

    const entityDrops = {
        'minecraft:wither': [
            { item: 'oresplus:wither_shard', amount: 3, prob: 100, condition: 1 },
            { item: 'oresplus:broken_gem', amount: 1, prob: 100, condition: 1 }
        ],
        'minecraft:ender_dragon': [
            { item: 'oresplus:alexandrite', amount: 9, prob: 100, condition: 1 },
            { item: 'oresplus:broken_gem', amount: 1, prob: 100, condition: 1 }
        ],
        'minecraft:wither_skeleton': [
            { item: 'oresplus:broken_gem', amount: 1, prob: 7, condition: 1 }
        ],
        'minecraft:husk': [
            { item: 'oresplus:essence_desert_gem', amount: 1, prob: 7, condition: 1 },
            { item: 'oresplus:broken_gem', amount: 1, prob: 7, condition: 1 }
        ],
        'minecraft:blaze': [
            { item: 'oresplus:essence_infernal_gem', amount: 1, prob: 7, condition: 1 },
            { item: 'oresplus:broken_gem', amount: 1, prob: 7, condition: 1 }
        ],
        'minecraft:shulker': [
            { item: 'oresplus:essence_end_gem', amount: 1, prob: 10, condition: 1 },
            { item: 'oresplus:broken_gem', amount: 1, prob: 7, condition: 1 }
        ],
        'minecraft:enderman': [
            { item: 'oresplus:essence_end_gem', amount: 1, prob: 7, condition: 1 },
            { item: 'oresplus:broken_gem', amount: 1, prob: 7, condition: 1 }
        ],
        'minecraft:elder_guardian': [
            { item: 'oresplus:essence_ocean_gem', amount: 3, prob: 100, condition: 1 },
            { item: 'oresplus:broken_gem', amount: 1, prob: 100, condition: 1 }
        ],
        'minecraft:phantom': [
            { item: 'arena:essence_angelic_gem', amount: 1, prob: 7, condition: 1 },
            { item: 'oresplus:broken_gem', amount: 1, prob: 7, condition: 1 }
        ],
        'minecraft:guardian': [
            { item: 'oresplus:essence_ocean_gem', amount: 1, prob: 7, condition: 1 },
            { item: 'oresplus:broken_gem', amount: 1, prob: 7, condition: 1 }
        ]
    };

    const drops = entityDrops[deadEntity.typeId];
    if (!drops) return;

    drops.forEach(drop => {
        if (!drop.condition) return;

        const randomChance = Math.random() * 100;
        if (randomChance > drop.prob) return;

        try {
            deadEntity.dimension.spawnItem(new ItemStack(drop.item, drop.amount), deadEntity.location);
        } catch {
            player.dimension.spawnItem(new ItemStack(drop.item, drop.amount), player.location);
        }
    });
});
