import { world, system, ItemStack } from '@minecraft/server'

system.runInterval(() => {
    let players = world.getAllPlayers();
    players.forEach(player => {
    if (player.getComponent('equippable').getEquipment('Feet')?.typeId == 'oresplus:cryo_boots') {
    player.runCommand("execute as @s at @s run fill ~+1 ~-1 ~-1 ~-1 ~-1 ~+1 packed_ice replace water");
    player.runCommand("execute as @s at @s run fill ~+1 ~-1 ~-1 ~-1 ~-1 ~+1 magma replace lava");
     }
 })
}, 10);


world.afterEvents.worldInitialize.subscribe(() => {
    system.runInterval(() => {
        // We obtain all the players 
        let players = world.getAllPlayers()
        players.forEach(player => {

            if (player.getComponent('equippable').getEquipment('Chest')?.typeId == 'oresplus:angelic_chestplate') {
        let jumps = player.getDynamicProperty('oresplus:jumps') || 0
        const maxExtraJumps = 10
        if (player.isFalling && jumps < maxExtraJumps) {
          if (player.isJumping) {
            let { x, y, z } = player.getVelocity()
            player.applyKnockback(x, z, Math.SQRT2 * Math.sqrt(x * x + z * z), 1)
            jumps += 1
          }
        }
        if (player.isOnGround) {
          jumps = 0
        }
        player.setDynamicProperty('oresplus:jumps', jumps)
      }
        });
    }, 1)
})


world.afterEvents.entityHurt.subscribe(e => {
    const { hurtEntity, damageSource, damage } = e
    const skeletons = [
        "minecraft:bogged",
        "minecraft:skeleton",
        "minecraft:skeleton_horse",
        "minecraft:stray",
        "minecraft:wither_skeleton",
    ];
    let attacker = damageSource.damagingEntity
    if (attacker?.typeId == 'minecraft:player') {

        if (attacker.getComponent('equippable').getEquipment('Mainhand')?.typeId == 'oresplus:cryo_sword') {
            hurtEntity.addEffect('slowness', 160, { amplifier: 4, showParticles: true });
        }
        if (attacker.getComponent('equippable').getEquipment('Mainhand')?.typeId == 'oresplus:wither_sword') {
            hurtEntity.addEffect('absorption', 40, { amplifier: 4, showParticles: true });
        }
        if (attacker.getComponent('equippable').getEquipment('Mainhand')?.typeId == 'oresplus:electro_sword') {
            hurtEntity.runCommand('execute as @s at @s run summon lightning_bolt ~ ~ ~');
        } 
        if (attacker.getComponent('equippable').getEquipment('Mainhand')?.typeId == 'oresplus:' && skeletons.includes(hurtEntity?.typeId)) {
            hurtEntity.addEffect('levitation', 60, { amplifier: 16, showParticles: true });
        }
        giveEffect(attacker, hurtEntity)
    }
})

function giveEffect(player, tags, entity) {
    let effects = {};
    let effectTimes = {}; // Almacenar tiempos de cada efecto

    for (const [tag, effectList] of Object.entries(tags)) {
        if (!player.hasTag(tag)) continue;

        effectList.forEach(({ effect, condition, level, effectTime }) => {
            if (condition) {
                // Suma niveles correctamente y actualiza tiempos
                effects[effect] = (effects[effect] || 0) + level; // Suma niveles directamente
                effectTimes[effect] = Math.max(effectTimes[effect] || 0, effectTime);
            }
        });
    }

    // Manejo de efectos en conflicto
    const conflictingEffects = [
        { effect1: 'poison', effect2: 'regeneration' },
        { effect1: 'weakness', effect2: 'strength' },
        { effect1: 'mining_fatigue', effect2: 'haste' },
        { effect1: 'slowness', effect2: 'speed' }
    ];

    conflictingEffects.forEach(({ effect1, effect2 }) => {
        const level1 = effects[effect1] || 0;
        const level2 = effects[effect2] || 0;

        if (level1 > 0 && level2 > 0) {
            if (level1 === level2) {
                // Si los niveles son iguales, ambos se cancelan
                delete effects[effect1];
                delete effects[effect2];
                delete effectTimes[effect1];
                delete effectTimes[effect2];
            } else {
                // Mantiene el efecto más fuerte
                const strongerEffect = level1 > level2 ? effect1 : effect2;
                const weakerEffect = level1 > level2 ? effect2 : effect1;
                effects[strongerEffect] = Math.abs(level1 - level2);
                delete effects[weakerEffect];
                delete effectTimes[weakerEffect];
            }
        }
    });

    // Aplica los efectos al jugador con los tiempos correctos
    for (const [effectName, level] of Object.entries(effects)) {
        const time = effectTimes[effectName] || 0; // Tiempo correspondiente
        entity.addEffect(effectName, time, { amplifier: level - 1, showParticles: false }); // Amplificador comienza en 0
    }
}